package actions

import (
	"github.com/gin-gonic/gin"

	"github.com/iapologizewhenimwrong/Vestmore_GO/translator"
	"github.com/iapologizewhenimwrong/Vestmore_GO/utils/basic"
	"github.com/iapologizewhenimwrong/Vestmore_GO/utils/log"
)

type ActionContext[PARAM any] struct {
	Lang   string
	GinCtx *gin.Context
	Param  *PARAM
}

func (ctx *ActionContext[PARAM]) Localize(MessageID translator.TrCode) (string, error) {
	return translator.Localize(MessageID, ctx.Lang, nil)
}

func (ctx *ActionContext[PARAM]) LocalizeWithData(MessageID translator.TrCode, MessageTemplateParam any) (string, error) {
	return translator.Localize(MessageID, ctx.Lang, MessageTemplateParam)
}

func (ctx *ActionContext[PARAM]) LocalizeEx(MessageID translator.TrCode, Langs ...string) (string, error) {
	if len(Langs) == 0 {
		return translator.LocalizeEx(MessageID, nil, ctx.Lang)
	}
	return translator.LocalizeEx(MessageID, nil, Langs...)
}

func (ctx *ActionContext[PARAM]) LocalizeExWithData(MessageID translator.TrCode, MessageTemplateParam any, Langs ...string) (string, error) {
	if len(Langs) == 0 {
		return translator.LocalizeEx(MessageID, MessageTemplateParam, ctx.Lang)
	}
	return translator.LocalizeEx(MessageID, MessageTemplateParam, Langs...)
}

var HandlersFuncRoutes map[string]gin.HandlerFunc = make(map[string]gin.HandlerFunc)

func init() {
	// func AccountForgetSmsCode(ctx *gin.Context, param *AccountForgetSmsCodeParam) (resp *basic.Response)
	HandlersFuncRoutes["account/forgetSmsCode"] = AccountForgetSmsCodeHandler
	// func AccountLoginWithEmailPassword(ctx *gin.Context, param *AccountLoginWithEmailPasswordParam) (resp *basic.Response)
	HandlersFuncRoutes["account/loginWithEmailPassword"] = AccountLoginWithEmailPasswordHandler
	// func AccountLoginWithTelephonePassword(ctx *gin.Context, param *AccountLoginWithTelephonePasswordParam) (resp *basic.Response)
	HandlersFuncRoutes["account/loginWithTelephonePassword"] = AccountLoginWithTelephonePasswordHandler
	// func AccountRegisterEmailCode(ctx *gin.Context, param *AccountRegisterEmailCodeParam) (resp *basic.Response)
	HandlersFuncRoutes["account/registerEmailCode"] = AccountRegisterEmailCodeHandler
	// func AccountRegisterSmsCode(ctx *gin.Context, param *AccountRegisterSmsCodeParam) (resp *basic.Response)
	HandlersFuncRoutes["account/registerSmsCode"] = AccountRegisterSmsCodeHandler
	// func BaseGetToken(ctx *gin.Context, param *BaseGetTokenParam) (resp *basic.Response)
	HandlersFuncRoutes["base/getToken"] = BaseGetTokenHandler
	// func MemberAlterPassword(ctx *gin.Context, param *MemberAlterPasswordParam) (resp *basic.Response)
	HandlersFuncRoutes["member/alterPassword"] = MemberAlterPasswordHandler
}

type AccountForgetSmsCodeParam struct {
	Action      string `json:"action" form:"action" binding:"-"`
	CountryCode string `json:"country_code" form:"country_code" binding:"required"`
	Telephone   string `json:"telephone" form:"telephone" binding:"required"`
	Token       string `json:"token" form:"token" binding:"-"`
}

func AccountForgetSmsCodeHandler(ctx *gin.Context) {
	var resp *basic.Response

	param := &AccountForgetSmsCodeParam{}
	err := ctx.ShouldBind(param)
	if err != nil {
		log.Println(err)
		resp = resp.Error(basic.ErrParamParse)
		ctx.JSON(200, resp)
		return
	}

	actx := &ActionContext[AccountForgetSmsCodeParam]{
		Lang:   basic.GetLangString(param),
		GinCtx: ctx,
		Param:  param,
	}

	resp = AccountForgetSmsCode(actx)
	if resp == nil {
		resp = resp.Error(basic.ErrRespNotNil)
	}
	ctx.JSON(200, resp)
}

type AccountLoginWithEmailPasswordParam struct {
	Password  string `json:"password" form:"password" binding:"-"`
	Randstr   string `json:"randstr" form:"randstr" binding:"-"`
	Sign      string `json:"sign" form:"sign" binding:"-"`
	Action    string `json:"action" form:"action" binding:"-"`
	Device    string `json:"device" form:"device" binding:"-"`
	Version   string `json:"version" form:"version" binding:"-"`
	AppMarket uint64 `json:"app_market" form:"app_market" binding:"-"`
	Email     string `json:"email" form:"email" binding:"-"`
	Timestamp int64  `json:"timestamp" form:"timestamp" binding:"-"`
	Token     string `json:"token" form:"token" binding:"-"`
}

func AccountLoginWithEmailPasswordHandler(ctx *gin.Context) {
	var resp *basic.Response

	param := &AccountLoginWithEmailPasswordParam{}
	err := ctx.ShouldBind(param)
	if err != nil {
		log.Println(err)
		resp = resp.Error(basic.ErrParamParse)
		ctx.JSON(200, resp)
		return
	}

	actx := &ActionContext[AccountLoginWithEmailPasswordParam]{
		Lang:   basic.GetLangString(param),
		GinCtx: ctx,
		Param:  param,
	}

	resp = AccountLoginWithEmailPassword(actx)
	if resp == nil {
		resp = resp.Error(basic.ErrRespNotNil)
	}
	ctx.JSON(200, resp)
}

type AccountLoginWithTelephonePasswordParam struct {
	Randstr     string `json:"randstr" form:"randstr" binding:"-"`
	Sign        string `json:"sign" form:"sign" binding:"-"`
	Telephone   string `json:"telephone" form:"telephone" binding:"-"`
	Version     string `json:"version" form:"version" binding:"-"`
	Token       string `json:"token" form:"token" binding:"-"`
	CountryCode string `json:"country_code" form:"country_code" binding:"-"`
	Password    string `json:"password" form:"password" binding:"-"`
	Action      string `json:"action" form:"action" binding:"-"`
	Device      string `json:"device" form:"device" binding:"-"`
	AppMarket   uint64 `json:"app_market" form:"app_market" binding:"-"`
	Email       string `json:"email" form:"email" binding:"-"`
	Timestamp   int64  `json:"timestamp" form:"timestamp" binding:"-"`
}

func AccountLoginWithTelephonePasswordHandler(ctx *gin.Context) {
	var resp *basic.Response

	param := &AccountLoginWithTelephonePasswordParam{}
	err := ctx.ShouldBind(param)
	if err != nil {
		log.Println(err)
		resp = resp.Error(basic.ErrParamParse)
		ctx.JSON(200, resp)
		return
	}

	actx := &ActionContext[AccountLoginWithTelephonePasswordParam]{
		Lang:   basic.GetLangString(param),
		GinCtx: ctx,
		Param:  param,
	}

	resp = AccountLoginWithTelephonePassword(actx)
	if resp == nil {
		resp = resp.Error(basic.ErrRespNotNil)
	}
	ctx.JSON(200, resp)
}

type AccountRegisterEmailCodeParam struct {
	Randstr   string `json:"randstr" form:"randstr" binding:"-"`
	Sign      string `json:"sign" form:"sign" binding:"-"`
	Action    string `json:"action" form:"action" binding:"-"`
	AppMarket int    `json:"app_market" form:"app_market" binding:"-"`
	Email     string `json:"email" form:"email" binding:"-"`
	Timestamp int64  `json:"timestamp" form:"timestamp" binding:"-"`
	Token     string `json:"token" form:"token" binding:"-"`
}

func AccountRegisterEmailCodeHandler(ctx *gin.Context) {
	var resp *basic.Response

	param := &AccountRegisterEmailCodeParam{}
	err := ctx.ShouldBind(param)
	if err != nil {
		log.Println(err)
		resp = resp.Error(basic.ErrParamParse)
		ctx.JSON(200, resp)
		return
	}

	actx := &ActionContext[AccountRegisterEmailCodeParam]{
		Lang:   basic.GetLangString(param),
		GinCtx: ctx,
		Param:  param,
	}

	resp = AccountRegisterEmailCode(actx)
	if resp == nil {
		resp = resp.Error(basic.ErrRespNotNil)
	}
	ctx.JSON(200, resp)
}

type AccountRegisterSmsCodeParam struct {
	Action      string `json:"action" form:"action" binding:"-"`
	CountryCode string `json:"country_code" form:"country_code" binding:"required"`
	Telephone   string `json:"telephone" form:"telephone" binding:"required"`
	Token       string `json:"token" form:"token" binding:"-"`
}

func AccountRegisterSmsCodeHandler(ctx *gin.Context) {
	var resp *basic.Response

	param := &AccountRegisterSmsCodeParam{}
	err := ctx.ShouldBind(param)
	if err != nil {
		log.Println(err)
		resp = resp.Error(basic.ErrParamParse)
		ctx.JSON(200, resp)
		return
	}

	actx := &ActionContext[AccountRegisterSmsCodeParam]{
		Lang:   basic.GetLangString(param),
		GinCtx: ctx,
		Param:  param,
	}

	resp = AccountRegisterSmsCode(actx)
	if resp == nil {
		resp = resp.Error(basic.ErrRespNotNil)
	}
	ctx.JSON(200, resp)
}

type BaseGetTokenParam struct {
	Action    string `json:"action" form:"action" binding:"-"`
	AppMarket string `json:"app_market" form:"app_market" binding:"-"`
	Lang      string `json:"lang" form:"lang" binding:"-"`
	Token     string `json:"token" form:"token" binding:"-"`
}

func BaseGetTokenHandler(ctx *gin.Context) {
	var resp *basic.Response

	param := &BaseGetTokenParam{}
	err := ctx.ShouldBind(param)
	if err != nil {
		log.Println(err)
		resp = resp.Error(basic.ErrParamParse)
		ctx.JSON(200, resp)
		return
	}

	actx := &ActionContext[BaseGetTokenParam]{
		Lang:   basic.GetLangString(param),
		GinCtx: ctx,
		Param:  param,
	}

	resp = BaseGetToken(actx)
	if resp == nil {
		resp = resp.Error(basic.ErrRespNotNil)
	}
	ctx.JSON(200, resp)
}

type MemberAlterPasswordParam struct {
	Action          string `json:"action" form:"action" binding:"-"`
	ConfirmPassword string `json:"confirm_password" form:"confirm_password" binding:"-"`
	NewPassword     string `json:"new_password" form:"new_password" binding:"-"`
	OldPassword     string `json:"old_password" form:"old_password" binding:"-"`
	Token           string `json:"token" form:"token" binding:"required"`
}

func MemberAlterPasswordHandler(ctx *gin.Context) {
	var resp *basic.Response

	param := &MemberAlterPasswordParam{}
	err := ctx.ShouldBind(param)
	if err != nil {
		log.Println(err)
		resp = resp.Error(basic.ErrParamParse)
		ctx.JSON(200, resp)
		return
	}

	actx := &ActionContext[MemberAlterPasswordParam]{
		Lang:   basic.GetLangString(param),
		GinCtx: ctx,
		Param:  param,
	}

	resp = MemberAlterPassword(actx)
	if resp == nil {
		resp = resp.Error(basic.ErrRespNotNil)
	}
	ctx.JSON(200, resp)
}
