package actions

import (
	"github.com/gin-gonic/gin"
	
	"{{.ModuleName}}/translator"
    "{{.ModuleName}}/utils/basic"
    "{{.ModuleName}}/utils/log"
)

type ActionContext[PARAM any] struct {
	Lang   string
	GinCtx *gin.Context
	Param  *PARAM
}

func (ctx *ActionContext[PARAM]) Localize(MessageID translator.TrCode) (string, error) {
	return translator.Localize(MessageID, ctx.Lang, nil)
}

func (ctx *ActionContext[PARAM]) LocalizeWithData(MessageID translator.TrCode, MessageTemplateParam any) (string, error) {
	return translator.Localize(MessageID, ctx.Lang, MessageTemplateParam)
}

func (ctx *ActionContext[PARAM]) LocalizeEx(MessageID translator.TrCode, Langs ...string) (string, error) {
	if len(Langs) == 0 {
		return translator.LocalizeEx(MessageID, nil, ctx.Lang)
	}
	return translator.LocalizeEx(MessageID, nil, Langs...)
}

func (ctx *ActionContext[PARAM]) LocalizeExWithData(MessageID translator.TrCode, MessageTemplateParam any, Langs ...string) (string, error) {
	if len(Langs) == 0 {
		return translator.LocalizeEx(MessageID, MessageTemplateParam, ctx.Lang)
	}
	return translator.LocalizeEx(MessageID, MessageTemplateParam, Langs...)
}

var HandlersFuncRoutes map[string]gin.HandlerFunc = make(map[string]gin.HandlerFunc)

func init() {
{{- range .ActionFunctions}}
    // func {{.FuncName}}(ctx *gin.Context, param *{{.ParamStruct.ParamStructName}}) (resp *basic.Response)
	HandlersFuncRoutes["{{.ActionName}}"] = {{.FuncName}}Handler
{{- end}}
}

{{- range .ActionFunctions}}
type {{.ParamStruct.ParamStructName}} struct {
	{{- range .ParamStruct.ParamFields}}
	{{.ParamNameCamel}} {{.ParamType}} `json:"{{.ParamName}}" form:"{{.ParamName}}" binding:"{{.ParamBinding}}"`
	{{- end}}
}

func {{.FuncName}}Handler(ctx *gin.Context) {
    var resp *basic.Response  

	param := &{{.ParamStruct.ParamStructName}}{}
	err := ctx.ShouldBind(param)
	if err != nil {
        log.Println(err)
		resp = resp.Error(basic.ErrParamParse)
		ctx.JSON(200, resp)
		return
	}

	actx := &ActionContext[{{.ParamStruct.ParamStructName}}]{
		Lang:   basic.GetLangString(param),
		GinCtx: ctx,
		Param:  param,
	}
 
    resp = {{.FuncName}}(actx)
	if resp == nil {
		resp = resp.Error(basic.ErrRespNotNil)
	}
	ctx.JSON(200, resp)
}
{{end}}