package basic

import (
	"github.com/iapologizewhenimwrong/Vestmore_GO/translator"
	"github.com/iapologizewhenimwrong/Vestmore_GO/utils/log"
)

// 全局返回的结构体
type Response struct {
	Data      interface{} `json:"data"`
	ErrorCode int         `json:"error_code"`
	ErrorText string      `json:"error_text"`
	IsSuccess bool        `json:"success"`
}

type ILocalize interface {
	Localize(MessageID translator.TrCode) (string, error)
}

func (resp *Response) Error(errcode *ErrorCode, Data ...interface{}) *Response {
	if resp == nil {
		resp = &Response{}
	}
	resp.ErrorCode = errcode.Code
	resp.ErrorText = errcode.Message
	resp.IsSuccess = false
	resp.setData(Data)
	log.Error(resp.ErrorText)
	return resp
}

func (resp *Response) ErrorErr(Code int, err error, Data ...interface{}) *Response {
	if resp == nil {
		resp = &Response{}
	}
	resp.ErrorCode = Code
	resp.ErrorText = err.Error()
	resp.IsSuccess = false
	resp.setData(Data)
	log.Error(resp.ErrorText)
	return resp
}

func (resp *Response) ErrorMsg(Code int, Message string, Data ...interface{}) *Response {
	if resp == nil {
		resp = &Response{}
	}
	resp.ErrorCode = Code
	resp.ErrorText = Message
	resp.IsSuccess = false
	resp.setData(Data)
	log.Error(resp.ErrorText)
	return resp
}

// 实现翻译返回错误的代码
func (resp *Response) ErrorTrCode(il ILocalize, trcode translator.TrCode, Data ...interface{}) *Response {
	if resp == nil {
		resp = &Response{}
	}

	trStr, err := il.Localize(trcode)
	if err != nil {
		return resp.ErrorErr(1, err)
	}

	return resp.ErrorMsg(1, trStr)
}

func (resp *Response) Success(Data ...interface{}) *Response {
	if resp == nil {
		resp = &Response{}
	}
	resp.IsSuccess = true
	resp.setData(Data)
	return resp
}

func (resp *Response) setData(Data []interface{}) {
	if len(Data) == 0 {
		resp.Data = []interface{}{}
		return
	}

	if len(Data) == 1 {
		resp.Data = Data[0]
	} else {
		resp.Data = Data
	}
}
